/*!
 * SessionVOC Session
 *
 * A session store, which uses an external server, namely a SessionVOC
 * (see http://www.worldofvoc.com) to store and manage the sessions. The
 * connector needs the node-sessionvoc-client.
 *
 *  DISCLAIMER
 * 
 *  Copyright 2010-2011 triagens GmbH, Cologne, Germany
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Copyright holder is triAGENS GmbH, Cologne, Germany
 */

/**
 * Module dependencies.
 */

var connect = require('connect'),
    Cookie = connect.session.Cookie;

/**
 * Exports.
 */

module.exports = Session;

/* -----------------------------------------------------------------------------
 * constructors
 * ----------------------------------------------------------------------------- */

/**
 * Initializes a new 'Session'.
 *
 * @param {String} sid
 * @param {Object} data
 * @api public
 */

function Session (store, sid, data) {
  var self = this;

  // create getter for the transData
  var transGetter = function (name) {
    return function () {
      return data.transData[name];
    };
  };

  var transSetter = function (name) {
    return function (value) {
      return data.transData[name] = value;
    }
  };

  for (var attr in data.transData) {
    self.__defineGetter__(attr, transGetter(attr));
    self.__defineSetter__(attr, transSetter(attr));
  }

  // create getter for the userData
  var userGetter = function (name) {
    return function () {
      return data.userData[name];
    };
  };

  var userSetter = function (name) {
    return function (value) {
      return data.userData[name] = value;
    }
  };

  for (var attr in data.userData) {
    self.__defineGetter__(attr, userGetter(attr));
    self.__defineSetter__(attr, userSetter(attr));
  }

  // define fixed attributes
  self.cookie = new Cookie();

  self.__defineGetter__('sid', function () { return data.sid; });
  self.__defineGetter__('uid', function () { return data.uid; });

  self._data = data;
  self._store = store;
}

/* -----------------------------------------------------------------------------
 * Session public methods
 * ----------------------------------------------------------------------------- */

/**
 * Destroys a session.
 *
 * @param {String} sess
 */

Session.prototype.destroy = function (error) {
  next = next || function(err) { if (err) { throw err; } };

  var self = this;

  self._store.deleteSessionVoc(self.sid,
                               function (deleted) {
                                 if (! deleted.deleted) {
                                   next(deleted);
                                 }
                                 
                                 self._data.sid = null;
                                 self._data.transData = {};
                                 
                                 self._data.uid = null;
                                 self._data.userData = {};
                                 
                                 next();
                               },
                               function (err) {
                                 next(err);
                               });
};

/**
 * Reloads a session.
 *
 * @param {String} sess
 */

Session.prototype.reload = function (next) {
  next = next || function(err) { if (err) { throw err; } };

  var self = this;

  self._store.readSessionVoc(self._data.sid,
                             function (data) {
                               self._data.sid = data.sid;
                               self._data.uid = data.uid;

                               self._data.transData = data.transData;
                               self._data.userData = data.userData;

                               next();
                             },
                             function () {
                              self._data.sid = null;
                              self._data.transData = {};

                              self._data.uid = null;
                              self._data.userData = {};

                              next();
                             },
                             function (err) {
                               next(err);
                             });
};

/**
 * Saves a session.
 *
 * @param {String} sess
 */

Session.prototype.save = function (next) {
  next = next || function(err) { if (err) { throw err; } };

  var self = this;

  self._store.updateSessionVoc(self._data.sid, self._data,
                               function (data) {
                                 self._data.sid = data.sid;
                                 self._data.uid = data.uid;

                                 self._data.transData = data.transData;
                                 self._data.userData = data.userData;

                                 next();
                               },
                               function (err) {
                                 next(err);
                               });
};
