/*!
 * SessionVOC Store
 *
 * A session store, which uses an external server, namely a SessionVOC
 * (see http://www.worldofvoc.com) to store and manage the sessions. The
 * connector needs the node-sessionvoc-client.
 *
 *  DISCLAIMER
 * 
 *  Copyright 2010-2011 triagens GmbH, Cologne, Germany
 * 
 *  Licensed under the Apache License, Version 2.0 (the "License");
 *  you may not use this file except in compliance with the License.
 *  You may obtain a copy of the License at
 * 
 *      http://www.apache.org/licenses/LICENSE-2.0
 * 
 *  Unless required by applicable law or agreed to in writing, software
 *  distributed under the License is distributed on an "AS IS" BASIS,
 *  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *  See the License for the specific language governing permissions and
 *  limitations under the License.
 * 
 *  Copyright holder is triAGENS GmbH, Cologne, Germany
 */

/**
 * Module dependencies.
 */

var sessionvocClient = require('sessionvoc-client'),
    Session = require('./sessionvoc-session');

/**
 * Exports.
 */

module.exports = SessionVocStore;

/**
 * Export the debugging flag.
 */

module.exports.debug = false;

/* -----------------------------------------------------------------------------
 * constructors
 * ----------------------------------------------------------------------------- */

/**
 * Initializes a new `SessionVocStore`. You can define an options array to specifiy
 * the host and port.
 *
 * @param {Object} options
 * @api public
 */

function SessionVocStore (options) {
  var host = options && options.host;
  var port = options && options.port;

  var self = this;

  self.description = 'Secure and reliable session store using a SessionVOC';
  self.client = sessionvocClient.createClient(host, port); 
  self.datainfo = null;

  // executes after datainfo is fetched. 
  self.client.on('ready', function(dinfo) { 
    self.datainfo = dinfo; 
  }); 
};

/* -----------------------------------------------------------------------------
 * SessionVocStore public methods
 * ----------------------------------------------------------------------------- */

/**
 * Creates a new session object.
 *
 * @param {String} sid
 * @param {Object} data
 * @param {Function} next(session)
 */

SessionVocStore.prototype.createSession = function (sid, data, next) {
  next(new Session(this, sid, data));
};

/**
 * Reads or creates a new Session object. If the session could be read, then
 * the function 'read' is called.  If the session could not be restored, but a
 * new session could be created then the function 'created' is called. In case
 * of an error, the 'error' function is called. If 'sid' is not given, then 
 * a new session (and sid) is created.
 *
 * @param {String} sid (optional)
 * @param {Function} created(session)
 * @param {Function} read(session)
 * @param {Function} error(err)
 * @api public
 */

SessionVocStore.prototype.loadSession = function (sid, created, loaded, error) {
  var self = this;

  if (typeof sid === 'function') {
    error = loaded;
    loaded = created;
    created = sid;
    sid = false;
  }

  if (sid) {
    self.readSessionVoc(sid,
                        function (result) {
                          self.createSession(sid, result, loaded);
                        },
                        function () {
                          self.createSessionVoc(function (sid, result) {
                                                  self.createSession(sid, result, created);
                                                },
                                                error);
                        },
                        error);
  }
  else {
    self.createSessionVoc(function (sid, result) {
                            self.createSession(sid, result, created);
                          },
                          error);
  }
};

/* -----------------------------------------------------------------------------
 * SessionVocStore private methods
 * ----------------------------------------------------------------------------- */

/**
 * Attempts to create a new session. If the session could be created, then the
 * continuation function 'created' is called with the newly created session. In
 * case of an error, the 'error' function is called.
 *
 * @param {Function} created(sid,data)
 * @param {Function} error(err)
 * @api private
 */

SessionVocStore.prototype.createSessionVoc = function (created, error) {
  this.client.createSession(function(err, sid, data) {
    if (err) {
      return error(err);
    }
    else {
      return created(sid, data);
    }
  });
}; 

/**
 * Attempts to read session by the given `sid`. If the session could be read,
 * then the continuation function 'read' is called with the newly read
 * session. If the session could not be read, then the function 'notFound' is
 * called. In case of an error, the 'error' function is called.
 *
 * @param {String} sid
 * @param {Function} read(result)
 * @param {Function} notFound()
 * @param {Function} error(err)
 * @api private
 */

SessionVocStore.prototype.readSessionVoc = function (sid, read, notFound, error) {
  this.client.readSession(sid, function(err, data) {
    if (err) {
      if (err.errorCode == 404) {
        return notFound();
      }
      else {
        return error(err);
      }
    }
    else {
      return read(data);
    }
  });
}; 

/**
 * Attempts to update a session. If the session could be updated, then the
 * continuation function 'updated' is called. In case of an error, the 'error'
 * function is called.
 *
 * @param {String} sid
 * @param {Object} data
 * @param {Function} update(result)
 * @param {Function} error(err)
 * @api private
 */

SessionVocStore.prototype.updateSessionVoc = function (sid, data, updated, error) {
  this.client.updateSession(sid, data, function(err, data) {
    if (err) {
      return error(err);
    }
    else {
      return updated(data);
    }
  });
}

/**
 * Attempts to delete a session. If the session could be deleted, then the
 * continuation function 'deleted' is called. In case of an error, the 'error'
 * function is called.
 *
 * @param {String} sid
 * @param {Function} update(result)
 * @param {Function} error(err)
 * @api private
 */

SessionVocStore.prototype.deleteSessionVoc = function (sid, deleted, error) {
  this.client.deleteSession(sid, function(err, data) {
    if (err) {
      return error(err);
    }
    else {
      return deleted(data);
    }
  });
}

/* -----------------------------------------------------------------------------
 * functions for use from the shell and debugging.
 * ----------------------------------------------------------------------------- */

/**
 * Log informational output if debug flag is set.
 */

function debuglog (msg) {
  if (module.exports.debug) {
    console.log('[sessionvoc-store] ' + msg);
  }
}
