/*!
 * This is a modification of the original connect/session handler by:
 *
 * Connect - session
 *
 * Copyright(c) 2010 Sencha Inc.
 * Copyright(c) 2011 TJ Holowaychuk
 * MIT Licensed
 *
 * The modifications are
 *
 * Copyright(c) 2011 triAGENS GmbH, Cologne, Germany
 * MIT Licensed
 */

/**
 * Module dependencies.
 */

var SessionVocStore = require('./sessionvoc-store'),
    connect = require('connect'),
    utils = connect.utils;

/**
 * Expose the middleware.
 */

exports = module.exports = sessionvoc;

/**
 * Export the debugging flag.
 */

exports.debug = false;

/* -----------------------------------------------------------------------------
 * sessionvoc middleware
 * ----------------------------------------------------------------------------- */

/**
 * Setup session store with the given `options`.
 *
 * Session data is _not_ saved in the cookie itself, however cookies are used,
 * so we must use the [cookieParser()](middleware-cookieParser.html) middleware
 * _before_ `sessionvoc()`.
 *
 * Examples:
 *
 *     connect.createServer(
 *         connect.cookieParser(),
 *         sessionvoc()
 *     );
 *
 * Options:
 *
 *   - `host`          host name running the SessionVOC
 *   - `port`          port to connect to
 *
 * ## req.session
 *
 *  To store or access session data, simply use the request property
 *  `req.session`, which is (generally) serialized as JSON by the store, so
 *  nested objects are typically fine. The layout of the session depends on the
 *  layout defined in the SessionVOC.
 *
 *       connect(
 *           connect.cookieParser(),
 *           sessionvoc(),
 *           function(req, res, next) {
 *             var sess = req.session;
 *             if (sess.message) {
 *               res.setHeader('Content-Type', 'text/html');
 *               res.write('<p>Message: ' + sess.message + '</p>');
 *               sess.message = null;
 *               res.end();
 *           }
 *           else {
 *             res.message = "DEMO";
 *             res.end('welcome to the session demo. refresh!');
 *           }
 *         }
 *       ).listen(3000);
 *
 * ## Session#destroy()
 *
 *  Destroys the session, removing `req.session`, will be re-generated next request.
 *
 *      req.session.destroy(function (err) {
 *        // cannot access session here
 *      });
 * 
 * ## Session#reload()
 *
 *  Reloads the session data.
 *
 *      req.session.reload(function (err) {
 *        // session updated
 *      });
 *
 * ## Session#save()
 *
 *  Save the session.
 *
 *      req.session.save(function(err) {
 *        // session saved
 *      });
 *
 * ## Session#cookie
 *
 *  Each session has a unique cookie object accompany it.
 *
 * @param {Object} options
 * @return {Function}
 * @api public
 *
 */

function sessionvoc (options){
  var options = options || {},
      key = options.key || 'SESSIONVOC',
      store = new SessionVocStore(options);

  return function session(req, res, next) {

    // self-awareness
    if (req.session) {
      return next();
    }

    // proxy writeHead() to Set-Cookie
    var writeHead = res.writeHead;

    res.writeHead = function (status, headers) {
      res.writeHead = writeHead;

      if (req.session) {
        var cookie = req.session.cookie;

        // only send secure session cookies when there is a secure connection.
        // proxySecure is a custom attribute to allow for a reverse proxy
        // to handle SSL connections and to communicate to connect over HTTP that
        // the incoming connection is secure.
        var secured = cookie.secure && (req.connection.encrypted || req.connection.proxySecure);

        if (secured || !cookie.secure) {
          res.setHeader('Set-Cookie', cookie.serialize(key, req.session.sid));
        }
      }

      return res.writeHead(status, headers);
    };

    // proxy end() to commit the session
    var end = res.end;

    res.end = function (data, encoding) {
      res.end = end;

      if (req.session) {

        // HACK: ensure Set-Cookie for implicit writeHead()
        if (!res._header) {
          res._implicitHeader();
        }

        req.session.save(function () {
          res.end(data, encoding);
        });
      }
      else {
        res.end(data, encoding);
      }
    };

    // get the session identifier from the cookie
    var sessionId = req.cookies[key.toLowerCase()];

    if (sessionId) {
      debuglog("found session identifier: " + sessionId);
    }
    else {
      debuglog("no cookie named '" + key + "'");
    }

    // load the session into the request
    var pause = utils.pause(req);

    var created = function (session) {
      req.session = session;

      if (session) {
        debuglog("session identifier is: " + session.sid);
      }

      next();
      pause.resume();
    };

    var error = function (err) {
      next(err);
      pause.resume();
    };

    store.loadSession(sessionId, created, created, error);
  };
};

/* -----------------------------------------------------------------------------
 * functions for use from the shell and debugging.
 * ----------------------------------------------------------------------------- */

/**
 * Log informational output if debug flag is set.
 */

function debuglog (msg) {
  if (exports.debug) {
    console.log('[sessionvoc] ' + msg);
  }
}
